/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper
 * Copyright (C) 1995  Ricky Beam, Rob Cox
 * Copyright (C) 1996  Joerg Weule
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/***************************************************************************
 *
 * a2dc.c
 *
 ***************************************************************************/

#include <linux/autoconf.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/major.h>
#include <linux/mm.h>
#include <linux/timer.h>
#include <linux/kernel.h>
#include <linux/malloc.h>
#include <linux/string.h>
#include <linux/signal.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <stdlib.h>

#define ADC_MODULE 1

#include "adc.h"

/*------------a2dc.h*--------------------------------------------------------*/
/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper  (CIO-DAS16/330)
 * Copyright (C) 1995  Ricky Beam, Rob Cox       (CIO-DAS1600)
 * Copyright (C) 1996  Joerg Weule               (CIO-DAS1600) Improved Buffer
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/*
 * a2dc.h
 */

/* Default device major number */
#ifndef DEFAULT_MAJOR_DEV
#define DEFAULT_MAJOR_DEV  31    /* Default Major Device Number */
#endif
#define DEFAULT_FREQ        50   /* Default frequency */
#define DAS_IRQ              7   /* Interrupt for read() 2-7 */
#define DAS_DMA              1   /* DMA Channel  */
#define AD_CHANNELS         16   /* 16 Single Ended or 8 Diff. */
#define DIO_CHANNELS         4   /* 4 Input and 4 Output */
#define DIO_PORTS            3   /* 24 Digital I/O Ports (3 bytes) */
#define DA_PORTS             2   /* 2 D/A ports */
#define BASE_REG         0x300   /* base address for CIO-DAS 1600 */

/* I/O Register locations */

#define LSB_AND_CHNLS      BASE_REG      /* A/D Data & Channel Register */
#define MSB_DATA_BYTE      BASE_REG+1    
#define MUX_SCAN_LIMITS    BASE_REG+2    /* Channel Mux Scan Limits Register */
#define DIO_REG            BASE_REG+3    /* 4 Bit Digital I/O Register */
#define DA0L               BASE_REG+4    /* D/A 0 Bits 0-3 */
#define DA0H               BASE_REG+5    /* D/A 0 Bits 4-11 Start of D/A */
#define DA1L               BASE_REG+6    /* D/A 1 Bits 0-3 */
#define DA1H               BASE_REG+7    /* D/A 1 Bits 4-11 Start of D/A */
#define STATUS_REG         BASE_REG+8    /* Status Register */
#define CNTRL_REG          BASE_REG+9    /* DMA, Interrupt & Trigger Control */
#define PACER_CLOCK_REG    BASE_REG+10   /* Pacer Clock Control Register */
#define MISC_REG           BASE_REG+11   /* Analog Input Range Reg., ETC */

#define COUNTERA_0_DATA    BASE_REG+12   /* Pacer Clock A Counter 0 Register */
#define COUNTERA_1_DATA    BASE_REG+13   /* Pacer Clock A Counter 1 Register */
#define COUNTERA_2_DATA    BASE_REG+14   /* Pacer Clock A Counter 2 Register */
#define COUNTERA_CONTROL   BASE_REG+15   /* Pacer Clock A Control  Register */

#define COUNTERB_0_DATA    BASE_REG+16   /* Pacer Clock B Counter 0 Register */
#define COUNTERB_1_DATA    BASE_REG+17   /* Pacer Clock B Counter 1 Register */
#define COUNTERB_2_DATA    BASE_REG+18   /* Pacer Clock B Counter 2 Register */
#define COUNTERB_CONTROL   BASE_REG+19   /* Pacer Clock B Control  Register */

#define DIO_PORTA          BASE_REG+0x400  /* Port A (  8  bit ) I/O */
#define DIO_PORTB          BASE_REG+0x401  /* Port B (  8  bit ) I/O */
#define DIO_PORTC          BASE_REG+0x402  /* Port C ( 4+4 bit ) I/O */
#define DIO_REG_EXTENDED   DIO_PORTA
#define DIO_CNTRL_REG      BASE_REG+0x403  /* Mode and Direction Control Register */

#define CONVERT_DIS_REG    BASE_REG+0x404  /* Prevent A/D trigger conversions */
#define BURST_MODE_REG     BASE_REG+0x405  /* DAS1600 Burst Mode Enable/Disable */
#define MODE_ENABLE_REG    BASE_REG+0x406  /* DAS1600 functions Enable/Disable */
#define BURST_STAT_REG     BASE_REG+0x407  /* DAS1600 Board Status Register    */

#define DAS1600_BME 1<<6
#define DAS1600_ME  1<<5
#define DAS1600_CD  1<<4
#define DAS1600_WS  1<<1
#define DAS1600_CLK 1<<0

#define DA_MINOR           24
#define DA_PORT0           BASE_REG+4

/* Errors */

#define BAD_CHANNEL        1
#define BAD_COUNT          2
#define BAD_GAIN           3
#define NO_ENHANCED_MODE   4
#define BAD_SPEED          5
#define ADC_BUSY           6
#define INT_ALREADY_ACTIVE 7

/* General defines */

#define TRUE  1
#define FALSE 0

/* 
#define DEBUG 
  */

/* DT Connection is not supported */
/* The 1600 cards have it alwasys on... */

#define DT_CONNECT FALSE


/*************************************************************************
* STATUS_PORT        BASE_REG+8             Status Register              *
**************************************************************************/
#define ADSR_INT        0x10      /* INT=1 External pulse received, INT=0 waiting */
#define ADSR_MUX        0x20      /* MUX=1 16 Single Ended, MUX=0 8 Differential */
#define ADSR_UB         0x40      /* U/B=1 Unipolar mode, U/B=0 Bipolar mode */
#define ADSR_EOC        0x80      /* EOC=1, A/D is busy, EOC=0 free */

/*************************************************************************
* CNTRL_REG          BASE_REG+9        DMA, Interrupt & Trigger Control  *
**************************************************************************/
#define CNTL_TS0       0x1        /* TS0=0 External Trigger, TS0=1 Pacer Clock */
#define CNTL_TS1       0x2        /* TS1=0 Software triggered A/D only */
#define CNTL_DMA       0x4        /* DMA Disabled = 0, DMA Enabled = 1 */
#define CNTL_INTE      0x80       /* Interrupts Disabled = 0, Enabled = 1 */

/*************************************************************************
* MISC_REG        BASE_REG+11         Range, and Exteded Features        *
**************************************************************************/

#define ENHANCED_BIT  (0x10)     /* Enhanced Mode Enabled = 1, Disabled = 0 */
#define MISC_OVERRUN  (0x20)     /* No overrun = 0, FIFO buffer full = 1 */
#define MISC_PRETRIG  (0x40)     /* Pretrigger Enabled = 1, Disable = 0 */
#define MISC_DT       (0x80)     /* DT-Connect Enable = 1, Disable = 0 */

/*************************************************************************
* Constants for dealing with the 8254 counters                           *
**************************************************************************/

#define MODE0 0x0
#define MODE1 0x2
#define MODE2 0x4
#define MODE3 0x6
#define MODE4 0x8
#define MODE5 0xa

#define C0 0x00
#define C1 0x40
#define C2 0x80

#define LATCH    0x00
#define LSBONLY  0x10
#define MSBONLY  0x20
#define LSBFIRST 0x30

#define S0           0x00
#define S1           0x02

/*------------a2dc.h*--------------------------------------------------------*/

/***************************************************************************
 *
 * Global data.
 *
 ***************************************************************************/

/* Interrupt context data. Controls interrupt service handler */

typedef struct ADC_REC {
  LONG freq;           /* Complete frequency value */
  WORD ctr1;           /* MSB of estimated "freq" */
  WORD ctr2;           /* LSB of estimated "freq" */
  BYTE pacerReg;       /* value of pacer clock control register */
} BoardRec;
static BoardRec BoardData;                  /* Board specific information */

/* Values kept. */

typedef struct CHANNEL_REC {
  int  open;            /* Is channel device open() */
  int  pacer;           /* Pacer and DMA is active */
  int  adc_dio_pretrig; /* TRUE = write dio before sampling                  */
  BYTE dio_value;       /* DIO value to be written before sampling           */
  long total;           /* Total words read from device                      */
  BYTE mode;            /* 0 = Compatibility Mode, Soft Trigger              */
                        /* 2 = Extended Mode,      External Trigger          */
                        /* 4 = Extended Mode,      Pacer Clock               */
                        /* 6 = Extended Mode,      Pacer Clock               */
                        /*                           External Trig Enable    */
  BYTE gain;            /* Voltage range */
  BYTE minor;           /* Minor number of device/channel number */
  BYTE highScan,
       lowScan;         /* Mux Scan limits (expt.)                           */
  int  nScan;           /* highScan - lowScan + 1                            */
} ChanRec;
static ChanRec Chan;                    /* Channel specific information   */

static int MajorNumber = DEFAULT_MAJOR_DEV; /* Major number compiled in   */

/* char kernel_version[] = UTS_RELEASE;*/        /* Required to by module */
struct wait_queue  *das0_wait = NULL ;              /* wait semaphore     */

/***************************************************************************
 *
 * Prototype of pubic and private functions.
 *
 ***************************************************************************/

static int  PacerReadBlocked(WORD*KernBuff,int WordCount);
static void SetGains(BYTE gain);
static int  SetPacerFreq(BoardRec *board);
static void StopPacer();
static void StartPacer();
static void SetChannelMux(BYTE lowChan, BYTE highChan);
static void LoadPacer(BoardRec *board);
static void LoadCounter0( LONG arg );

static int das_read(struct inode *iNode, struct file *filePtr,
		    char *buf, int count);
static int das_write(struct inode *iNode, struct file *filePtr,
		     const char *buf, int count);
static int das_open(struct inode *iNode, struct file *filePtr);
static void das_close(struct inode *iNode, struct file *filePtr);
static int das_ioctl(struct inode *iNode, struct file *filePtr,
		     unsigned int cmd, LONG arg);
static void das0_ReadInterrupt(int);
static void doDMA(void);
static void buf_check_len(void);

int init_module(void);
void cleanup_module(void);

/*===========================================================================*/
/***************************************************************************
 *
 * Code for DMAbuffer
 *
 ***************************************************************************/

/*
 * Copyright (C) 1996 Dr. Joerg Weule
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/***************************************************************************
 *
 * Buffer to use DMA.
 *
 ***************************************************************************/

/* The buffer is used as a ring of cells, each is filled with DMA. After
 * a trasfer is complete, the next cell is installed for DMA.
 *
 * This idea needs the length of the buffer to be a multiple of the length of
 * DMA, which is a multiple of the length of the number of channels used.
 *
 * We call these length buf_len and buf_dma_len respectively.
 *
 * To be able to see if the buffer is full or empty, the buffer may not
 * be completely full.
 *
 * Buffer with data:
 *        -------------------------------------------------------------
 *       |..............ddddddddddddddddddddddddddd....................|
 *        -------------------------------------------------------------
 *                      ^                          ^
 *                   buf_a                       buf_b
 *
 * The concept and coding of the buffer was done by Dr. J"org Weule
 *                                                  weule@uni-duesseldorf.de
 */

#define BUF_RETRY_LIMIT 3
#define BUF_SIZE ( 16 * 1024 )

static WORD *DMAbuf;                        /* static DMA buffer space   */
static WORD *buf_p;                         /* DMAbuf + buf_b */
static buf_dma_len = 1024 ;                 /* length up to an interrupt */
static volatile int buf_a = 0 ;             /* DMS uses the Buffer as a ring */
static volatile int buf_b = 0 ;
static volatile int buf_len = BUF_SIZE ;
static volatile int buf_wait;               /* driver is sleeping */
static volatile int buf_blocked;            /* driver is sleeping */

#define ADC_BUF_INIT disable_dma(DAS_DMA), \
                     buf_a = buf_b = 0 , \
                     buf_dma_len = 1024 , \
                     buf_len = BUF_SIZE , \
                     buf_wait = buf_blocked = FALSE  

/***************************************************************************
 *
 * Set DMA Length
 *
 ***************************************************************************/

static inline void buf_set_dma_len(LONG arg){
    if ( arg == 0 ) return ;
    buf_dma_len = arg ;

#ifdef DEBUG
    printk("%s: ioctl(ADC_SET_DMA_LEN) %d\n", ADAPTER_ID, (int) buf_dma_len);
#endif

}

/***************************************************************************
 *
 *Set buf_len and check all the buf_..._len variables
 *
 ***************************************************************************/

static inline void buf_check_len(){

    int nScan = Chan.highScan - Chan.lowScan + 1 ;
    if ( nScan < 1 ) nScan = 1 ;
#ifdef DEBUG
    printk("%s: buf_check_len(): nScan        = %d\n)", ADAPTER_ID,(int)nScan);
#endif

    if ( buf_dma_len > ( BUF_SIZE >> 2 )) buf_dma_len = BUF_SIZE  >> 2 ;

    buf_dma_len /= nScan ;
    if ( buf_dma_len == 0 ) buf_dma_len = 1 ;
    buf_dma_len *= nScan ;

    buf_len = BUF_SIZE ;
    buf_len /= buf_dma_len ;
    buf_len *= buf_dma_len ;

#ifdef DEBUG
    printk("%s: buf_check_len(): buf_dma_len  = %d\n)", ADAPTER_ID,(int)buf_dma_len);
#endif
}

/***************************************************************************
 *
 * Read Data
 *
 ***************************************************************************/

static inline int buf_read(WORD*p,int n){

#ifdef DEBUG
	printk("%s: R: Entering buf_read().\n",ADAPTER_ID);
#endif

	/*
	 * Look for latest samples.
	 */
	/*
	 * Copy the samples out.
	 */
	if ( buf_a != buf_b ){
		int a = buf_a ;
		int b = buf_b ;
		int k ;
		if ( b > a ) {
			k = b - a ;
			if ( k > n ) k = n ;
			memcpy_tofs(p,DMAbuf+a,k*sizeof(WORD));
			a += k ;
			if ( a >= buf_len ) a = 0 ;
			buf_a = a ;
			return k ;
		} else {
			k = buf_len - a ;
			if ( k >= n ) {
				memcpy_tofs(p,DMAbuf+a,n*sizeof(WORD));
				a += n ;
				if ( a >= buf_len ) a = 0 ;
				buf_a = a ;
				return n ;
			} else {
				/* fist use count k, than l, return k + l */
				int l = b ;
				memcpy_tofs(p,DMAbuf+a,k*sizeof(WORD));
				p += k ;
				n -= k ;
				if ( l > n ) l = n ;
				memcpy_tofs(p,DMAbuf,l*sizeof(WORD));
				a = l ;
				buf_a = l ;
				return k + l ;
			}
		}
	}
	return 0;
}

/***************************************************************************
 *
 * Reset Buffer
 *
 ***************************************************************************/

static inline void buf_reset(void){
	buf_b = buf_a = 0 ;
	buf_p = DMAbuf ;
}

/***************************************************************************
 *
 * Next DMA
 *
 ***************************************************************************/

static inline void buf_next(void){
	int a , b , n ;
#ifdef DEBUG
	printk("%s: I: buf_next()\n", ADAPTER_ID);
#endif
	a = buf_a ;
	b = buf_b ;
	buf_b = b = ( b + buf_dma_len ) % buf_len ;
	buf_p = DMAbuf+b;
	n = ( b - a ) % buf_len ;
	if ( n > ( buf_len - 2 * buf_dma_len )) {
		a = buf_a ;
		printk("%s: Buffer overrun !!!!!!!!!!!!!!!!!!\n",ADAPTER_ID);
		a = ( a + buf_dma_len ) % buf_len ;
		buf_a = a ;
	}
}

/***************************************************************************
 *
 * Clear DMA area
 *
 ***************************************************************************/

static inline void buf_clear(void){
	int i = buf_dma_len ;
	WORD*p = buf_p ;
#ifdef DEBUG
	printk("%s: ?: buf_clear()\n", ADAPTER_ID);
#endif
	do *p++ = 0x3aaf ; while ( --i );
}

/***************************************************************************
 *
 * Set DMA
 *
 ***************************************************************************/

static inline void buf_load_dma_controller(void){
    /* The DMA controller has to be setup before the DAS-1600 is told
       to use DMA transfer mode.  So, setup the DMA first.  DMA controller
       must be off before it can be setup.                                 */
    buf_clear();
#ifdef DEBUG
	printk("%s: ?: buf_load_dma_controller()\n", ADAPTER_ID);
	printk("%s: ?: buf_start_dma: buf_p = %p, Count = %d\n", ADAPTER_ID,
	  buf_p, buf_dma_len * sizeof(WORD));
#endif
    disable_dma(DAS_DMA);
    clear_dma_ff(DAS_DMA);
    set_dma_mode(DAS_DMA, DMA_MODE_READ);
    set_dma_addr(DAS_DMA, (long) buf_p);
    set_dma_count(DAS_DMA, buf_dma_len * sizeof(WORD));
    enable_dma(DAS_DMA);
}

/***************************************************************************
 *
 * DMA post processing
 *
 ***************************************************************************/

static inline void buf_dma_postprocessing(void){
#ifdef DEBUG
	printk("%s: I: buf_dma_postprocessing()\n", ADAPTER_ID);
#endif
	buf_next();
}

/***************************************************************************
 *
 * DMA post processing
 *
 ***************************************************************************/

static inline void ext_dma_postprocessing(void){
}

/*==========================================================================*/

/***************************************************************************
 *
 * Read Data
 *
 ***************************************************************************/

static int inline PacerReadBlocked(WORD*KernBuff,int n){
  int i , r ;
#ifdef DEBUG
  printk("%s: R: PacerReadBlocked(%d) ---v---\n",ADAPTER_ID,n);
#endif
  if ( n <= 0 ) return 0 ;
  r = 0 ;
  do {
    i = buf_read(KernBuff,n);
    (WORD*)KernBuff += i ;
    r += i ;
    n -= i ;
    if ( n <= 0 ) break ;
    cli();
#ifdef DEBUG
    printk("%s: sleep_on\n",ADAPTER_ID);
#endif
    buf_wait = TRUE ;
    interruptible_sleep_on(&das0_wait);
    /*
     * Interrupted ?
     */
    if ( buf_wait == TRUE ) break ;
  } while ( 1 ) ;
#ifdef DEBUG
  printk("%s: R: PacerReadBlocked(%d) ---^---\n",ADAPTER_ID,n);
#endif
  return r ;
}

/***************************************************************************
 *
 * read() service function
 *
 ***************************************************************************/

static int das_read(struct inode *iNode, struct file *filePtr, char *buf, int count)
{
  int stat;
  int n ;
  
#ifdef DEBUG
  printk("%s: R: das_read(): Entering das_read() vvvvvvvvvv\n",ADAPTER_ID);
#endif
  
  /* Check that data can be written to file. */
    
  if ((stat = verify_area(VERIFY_WRITE, buf,
		    sizeof(WORD) * count)) != 0) {
    printk("%s: R: das_read: Failed VERIFY_WRITE\n",ADAPTER_ID);
    return -1;
  }
  /* Area is secure... */

  if ( Chan.pacer == FALSE ) StartPacer();
    
  /* Read */
  count /= sizeof(WORD);
  n = ( buf_blocked )?
    PacerReadBlocked((WORD*)buf,count) :
    buf_read((WORD*)buf,count);
  Chan.total += n ;
  n *= sizeof(WORD);

#ifdef DEBUG
  printk("%s: R: das_read(): Leaving das_read() ^^^^^^^^^^^^\n",ADAPTER_ID);
#endif
  
  return(n);
}

/***************************************************************************
 *
 * write() service function
 *
 ***************************************************************************/

static int das_write(struct inode *iNode, struct file *filePtr, const char *buf, int count)
{
  return(-EINVAL);
}


/***************************************************************************
 *
 * open() service handler
 *
 ***************************************************************************/

static int das_open(struct inode *iNode, struct file *filePtr)
{
  /* 
     check if device is already open: only one process may read from a
     port at a time.  There is still the possibility of two processes 
     reading from two different channels messing things up. However,
     the overhead to check for this may not be worth it.
     */
  
  if ( Chan.open == TRUE ) {
    return -EBUSY;
  }
  
  MOD_INC_USE_COUNT;
  
  Chan.open = TRUE;	/* The device is open */
  Chan.pacer = FALSE;	/* Pacer and DMA not active */
  Chan.gain = BP_10_00V;	/* +/- 10V */
  Chan.mode = filePtr->f_flags; /* set trigger mode */
  Chan.highScan = (BYTE) 0;
  Chan.lowScan = (BYTE) 0;
  Chan.nScan = (int) 0;
  Chan.total = (long) 0;
  
  SetChannelMux(0,0);

  BoardData.pacerReg = 0x0;
  
  outb_p(0x00, STATUS_REG);	/* Clear INT bit */
  outb_p(0x0, MISC_REG);	/* Put in compatibility mode */
  {
    BYTE bReg ;
    bReg = inb_p(CNTRL_REG);
    bReg &= ~(CNTL_INTE | CNTL_DMA);
    outb_p(bReg, CNTRL_REG);
  }
  ADC_BUF_INIT ;
 
	  
	#ifdef DEBUG
	  printk("%s: das_open() end ...\n",ADAPTER_ID);
	#endif

  return 0;   
}

/***************************************************************************
 *
 * close() service handler
 *
 ***************************************************************************/

static void das_close(struct inode *iNode, struct file *filePtr)
{
  outb_p(0x00, STATUS_REG);	/* Clear INT bit */

  StopPacer() ;
#ifdef DEBUG
  printk("%s: Words read: %ld.\n",ADAPTER_ID,(long)Chan.total);
  printk("%s: close().\n",ADAPTER_ID);
#endif
  
  Chan.open = FALSE;
  MOD_DEC_USE_COUNT;

}

/***************************************************************************
 *
 * iotctl() service handler
 *
 ***************************************************************************/

/* Note:
   Remember that FIOCLEX, FIONCLEX, FIONBIO, and FIOASYN are reserved ioctl cmd numbers
   */

static int das_ioctl(struct inode *iNode, struct file *filePtr, unsigned int cmd, LONG arg)
{
  switch (cmd) {
  case ADC_START_PACER:
    StartPacer();
  break;

  case ADC_SET_GAINS:
    Chan.gain = (BYTE)arg;
#ifdef DEBUG
    printk("%s: ioctl(ADC_SET_GAINS) %#x\n", ADAPTER_ID, (int) arg);
#endif
    break;

  case ADC_DA0:
    {
      BYTE l = ((unsigned long)arg) << 4 ;
      BYTE h = ((unsigned long)arg) >> 4 ;
      outb_p(l,DA0L);
      outb_p(h,DA0H);
    }
#ifdef DEBUG
    printk("%s: ioctl(ADC_DA0) %ld\n", ADAPTER_ID, (long) arg);
#endif
    break ;

  case ADC_DA1:
    {
      BYTE l = arg << 4 ;
      BYTE h = arg >> 4 ;
      outb_p(l,DA1L);
      outb_p(h,DA1H);
    }
#ifdef DEBUG
    printk("%s: ioctl(ADC_DA1) %ld\n", ADAPTER_ID, (long) arg);
#endif
    break ;
    
  case ADC_DO_BLOCK:
    buf_blocked = TRUE ;
    break ;

  case ADC_SET_DMA_LEN:
    buf_set_dma_len(arg);
    break ;

  case ADC_GET_DMA_LEN:
    buf_check_len();
    cli();
    put_fs_long(buf_dma_len, (long*) arg);
    sti();
    break;
    
  case ADC_SET_PACER_FREQ:
#ifdef DEBUG
    printk("%s: ioctl(ADC_SET_PACER_FREQ) %ld\n", ADAPTER_ID, (long) arg);
#endif
    if ( BoardData.freq > MAX_FREQ  ) {
      printk("%s: ioctl:  BoardData.freq = %ld out of range.\n",ADAPTER_ID,
	     BoardData.freq);
      return -1;
    } else {
      BoardData.freq = (LONG)arg;
      SetPacerFreq(&BoardData);
      LoadPacer(&BoardData);	/* load the board frequency now */
      outb_p(BoardData.pacerReg, PACER_CLOCK_REG);  
    }
    break;
    
  case ADC_GET_PACER_FREQ:
    cli();
    put_fs_long(BoardData.freq, (long*) arg);
    sti();
    break;
    
  case ADC_STOP_PACER:
    StopPacer();
    break;
    
  case CTR0:
    if ( arg == 1 ) {
      BoardData.pacerReg |= 0x2;
    } else {
      BoardData.pacerReg &= ~(0x2);
    }
    outb_p(BoardData.pacerReg, PACER_CLOCK_REG);  
    break;
    
  case COUNTER0:
    LoadCounter0( (LONG) arg );
    break;
    
  case ADC_DIO_PRESET:
    if ( arg == -1 ) {
      Chan.adc_dio_pretrig = FALSE;
    } else {
      Chan.adc_dio_pretrig = TRUE;
      Chan.dio_value = (BYTE) (arg & 0xf);
    }
    
#ifdef DEBUG
    printk("%s: ioctl(ADC_DIO_PRESET):  value = %#x\n",ADAPTER_ID,
      Chan.dio_value);
#endif
    
    break;
    
  case ADC_SETMUX_LIMITS:
    Chan.highScan = (arg & 0xf);
    Chan.lowScan = (arg & 0xf0) >> 4 ;
    SetChannelMux(Chan.lowScan,Chan.highScan);
    if (Chan.highScan > AD_CHANNELS) {
      Chan.highScan = 0;
      BoardData.pacerReg &= 0x3;
      return(-EINVAL);
    }
    Chan.nScan = Chan.highScan - Chan.lowScan ;
    BoardData.pacerReg |= Chan.nScan << 4 ;
    Chan.nScan += 1 ;
    outb_p(BoardData.pacerReg, PACER_CLOCK_REG);  
#ifdef DEBUG
    printk("%s: ioctl(ADC_SETMUX_LIMITS):  value = %#x\n",ADAPTER_ID, (int)arg);
    printk("%s: ioctl(ADC_SETMUX_LIMITS):  low = %#x  high = %#x\n",
        ADAPTER_ID,
        (int)Chan.lowScan,(int)Chan.highScan);
#endif
    break;
    
  case ADC_SET_MODE:
    Chan.mode = (BYTE) arg;
    BoardData.pacerReg &= ~(0x1);
    break;

  default:
    return(-EINVAL);
    break;
  }

  return 0;
}

/***************************************************************************
 *
 *
 ***************************************************************************/

static struct file_operations das_fops = {
  NULL,				/* seek    */
  das_read,			/* read    */
  das_write,			/* write   */
  NULL,				/* readdir */
  NULL,				/* select  */
  das_ioctl,			/* ioctl   */
  NULL,				/* select  */
  das_open,			/* open    */
  das_close,			/* close   */
  NULL				/* fsync   */
};

/***************************************************************************
 *
 * Loads driver. Called when "insmod adc.o" is invoked on the command line.
 *               The board is set to IRQ, 
 *
 ***************************************************************************/

int init_module(void) 
{
  register int i, retry;
  register BYTE bReg;
#ifdef DEBUG
  register int j;
#endif

  retry = 0;
  do {
    DMAbuf = (WORD*) kmalloc(BUF_SIZE * sizeof(WORD), GFP_DMA);
  } while ((DMAbuf == NULL) && (retry < BUF_RETRY_LIMIT));

  if (!DMAbuf ) {
    printk("%s: Unable to allocate DMA buffers of %d bytes\n",
	   ADAPTER_ID, BUF_SIZE * sizeof(WORD));
    return -EIO;
  }

  /* Register as a device with kernel.  */

  if (register_chrdev(MajorNumber, ADAPTER_ID, &das_fops)) {
    printk("%s: Unable to register character device.\n", ADAPTER_ID);
    return -EIO;
  } else {
    printk("%s: BASE=%#x IRQ=%d DMA=%d DMABufsiz= %d words\n",
	   ADAPTER_ID, BASE_REG, DAS_IRQ, DAS_DMA, BUF_SIZE);
    printk("%s: DMABufsiz=%#x [%p-%p]\n",
	   ADAPTER_ID, BUF_SIZE * sizeof(WORD),
           DMAbuf, DMAbuf+BUF_SIZE);
    printk("%s:  4/27/95 wjasper@tx.ncsu.edu sam@tx.ncsu.edu\n",ADAPTER_ID);
    printk("%s: 10/13/95 jfbeam@tx.ncsu.edu rncox@tx.ncsu.edu\n",ADAPTER_ID);
    printk("%s: 01/04/96 weule@uni-duesseldorf.de\n",ADAPTER_ID);
  }

  cli();

  /* Turn out interrupt before enabling it */ 
  /* Set interrupt level on board */
  bReg = DAS_IRQ;
  bReg <<= 4;			/* Tell ADC which IRQ to use */
  outb_p(bReg, CNTRL_REG);	/* ... clearing INTE bit in control reg */

  /* Register interrupt handler. */
  if (request_irq(DAS_IRQ, (void*)das0_ReadInterrupt, SA_INTERRUPT, ADAPTER_ID,NULL)) {
    sti();
    printk("%s: Can't request IRQ %d\n", ADAPTER_ID, DAS_IRQ);
    return -(EIO);
  }

  /* Register DMA. */
  if (request_dma(DAS_DMA, ADAPTER_ID)) {
    sti();
    printk("%s: Can't request DMA %d\n", ADAPTER_ID, DAS_DMA);
    free_irq(DAS_IRQ,NULL);
    return -(EIO);
  }

  /* Set all channel structures to show nothing active/open */
  for (i = 0; i < AD_CHANNELS; i++) {
    Chan.open = FALSE;
    Chan.adc_dio_pretrig = FALSE;
  }

#ifdef DEBUG
  for (i = 0, j = 0; i < BUF_SIZE; i++) {
    DMAbuf[i] = (WORD) j++;
  }

  printk("%s: DMAbuf == %p  \n",ADAPTER_ID, DMAbuf);
#endif

  das0_wait = NULL;

  BoardData.freq = DEFAULT_FREQ; /* Set default pacer clock frequency */
  SetPacerFreq(&BoardData);
  outb_p(0x0, MISC_REG);	/* Put in compatibility mode */

  outb_p(0x40, MODE_ENABLE_REG); /* DAS1600 Functions ENABLED */
  outb_p(0x0, BURST_MODE_REG);	/* Burst Mode DISABLED        */
  outb_p(0x0, CONVERT_DIS_REG);	/* Conversions are ENABLED   */

  bReg = inb_p(BURST_STAT_REG);
  printk("%s:     Burst Mode %s\n", ADAPTER_ID,
	 bReg&DAS1600_BME?"Enabled":"Disabled");
  printk("%s:     DAS1600 Mode %s\n", ADAPTER_ID,
	 bReg&DAS1600_ME?"Enabled":"Disabled");
  printk("%s:     Conversions %s\n", ADAPTER_ID,
	 bReg&DAS1600_CD?"Allowed":"Blocked");
  printk("%s:     WaitState %s\n", ADAPTER_ID,
	 bReg&DAS1600_WS?"Enabled":"Disabled");
  printk("%s:     Using %dMHz Clocks\n", ADAPTER_ID,
	 bReg&DAS1600_CLK?10:1);

  sti();

#ifdef DEBUG
  printk("%s: Setup complete. IRQ set to %d\n", ADAPTER_ID, DAS_IRQ);
#endif

  return 0;
}

/***************************************************************************
 *
 * Remove driver. Called when "rmmod adc" is run on the command line.
 *
 ***************************************************************************/

void cleanup_module(void) 
{
  if (MOD_IN_USE) {
    printk("%s: device busy, remove delayed\n", ADAPTER_ID);
    return;
  }

  /* Release the DMA channel and the IRQ */
  free_dma(DAS_DMA);
  free_irq(DAS_IRQ,NULL);

  /* Return the DMA buffers to free memery */
  if ( DMAbuf ) kfree(DMAbuf);

  if (unregister_chrdev(MajorNumber, ADAPTER_ID) != 0) {
    printk("%s: Unable to unregister character device.\n", ADAPTER_ID);
  } else {
    printk("%s: module removed.\n\n", ADAPTER_ID);
  }    
}

/***************************************************************************
 *
 * Interrupt handler used to service enhanced mode(interrupt) read().
 *
 ***************************************************************************/

void das0_ReadInterrupt(int irq)
{
  cli();
#ifdef DEBUG
  printk("%s: I: das0_ReadInterrupt(%d) v v v v v v v v v v \n",ADAPTER_ID,irq);
#endif
  { BYTE bReg ;
    bReg = inb_p(STATUS_REG);
    if (( bReg & ADSR_INT ) == 0 ){
      printk("%s: I: ERR: das0_ReadInterrupt(%d) why ? Status_Reg = %#x\n",
        ADAPTER_ID,irq,bReg);
      return ;
    }
  }
  if ( *buf_p == 0x3aaf ) {
    printk("%s: I: ERR: das0_ReadInterrupt(%d) why ? DMAbuf[buf_b] = %#x\n",
      ADAPTER_ID,irq,*buf_p);
    return ;
  }

  /*
   * Now we have to set up the variables for the next DMA transfer.
   */
  if ( Chan.pacer == TRUE ) {
    ext_dma_postprocessing();
    buf_dma_postprocessing();
    if ( buf_wait == TRUE ) {
      buf_wait = FALSE ;
#ifdef DEBUG
      printk("%s: I: wakeup \n",ADAPTER_ID);
#endif
      wake_up_interruptible(&das0_wait);
    }
  }
  /*
   * May be we turn off interrupt and DMA but one is still in the system.
   * Chan.pacer is FALSE than.
   * We test this case now.
   */
  if ( Chan.pacer == TRUE ) doDMA();
#ifdef DEBUG
  printk("%s: I: das0_ReadInterrupt(%d) ^ ^ ^ ^ ^ ^ ^ ^ ^ ^ \n",ADAPTER_ID,irq);
#endif
  sti();
}

/***************************************************************************
 *
 * Set voltage gains.
 *
 ***************************************************************************/

static void SetGains( BYTE gain )
{
  BYTE mask;
  BYTE test;
  
  mask = inb_p(MISC_REG);	/* read the Analog Input Range Register */
  mask &= 0xf0;			/* mask off last 4 bits                 */
  mask |= gain;			/* set gain                             */
  outb_p(mask, MISC_REG);
  test = inb_p(MISC_REG);
#ifdef DEBUG
  printk("%s:     Gain Setting:  %#x \n", ADAPTER_ID, gain); 
#endif
  return;
}

/****************************************************************************
 *   Set Pacer Clock Frequency
 * 
 *   Description: 
 *       Set the counters so that the pacer clock runs at the
 *       desired frequency.  The frequency is generated by dividing
 *       down a 10 MHz clock, so all frequencies can not be generated.
 *       This routine calculated the divisor to generate a frequency
 *       as near as possible to the requested one.  It then calculates
 *       the real frequency and returns it .
 ****************************************************************************/

static int SetPacerFreq(BoardRec *board)
{
  short unsigned ctr1, ctr2;
  LONG product, error;

  /* divide 10Mhz by frequency */

  /* This assumes the board is set for 10Mhz -- RB */

  product =  10000000.0 / board->freq + 0.5;

  /*
     Now the job is to find two 16 bit numbers, that when multiplied
     together are approximately equal to product.  Start by setting
     one of them, ctr1 to 2 (minimum settable value) and increment until
     the error is minimized and ctr2 is less than 32768.
     
     NOTE: In Mode 2, a value of 1 is illegal! Therefore, crt1 and crt2
     can never be 1.
     */

  ctr1 = product / 32768;
  if ( ctr1 < 2 ) ctr1 = 2;
  ctr2 = product / ctr1;
  error = abs(product - ctr2*ctr1);

  while ( error && ctr1 < 32768  && ctr2 > 1 ) {
    ctr1++;
    ctr2 = product / ctr1;
    error = abs(product - ctr2*ctr1);
  }

  /* the frequency is prime, add 1 to it */

  if ( error ) {
    product++;
    ctr1 = product / 32768;
    if ( ctr1 < 2 ) ctr1 = 2;
    ctr2 = product / ctr1;
    error = abs(product - ctr2*ctr1);

    while ( error  && ctr1 < 32768 && ctr2 > 1) {
      ctr1++;
      ctr2 = product / ctr1;
      error = abs(product - ctr2*ctr1);
    }
  }

  /* we can't have ctr2 equal to 1, or system hangs */

  if ( ctr2 == 1 ) {
    ctr2++;
    ctr1 /= 2;
  }

  board->ctr1 = ctr1 ;
  board->ctr2 = ctr2 ;
  board->freq = 10000000.0/(ctr1*ctr2) + 0.5;
#ifdef DEBUG
  printk("%s: SetPacerFreq: A ctr1 %d ctr2 %d\n",ADAPTER_ID,
	 board->ctr1, board->ctr2);
#endif

#ifdef DEBUG
  printk("%s: SetPacerFreq: Pacer Register set to %#x\n",
    ADAPTER_ID, BoardData.pacerReg);
#endif

  return 0;
}

/***************************************************************************
 *
 * Load two part frequency to pacer counter chip.
 *
 ***************************************************************************/

static void LoadPacer(BoardRec *board)
{
  BYTE mask;
  BYTE bData;

  /* Write the values of ctr1 and ctr2 into counter A1 and A2 */

  cli();

#ifdef DEBUG
  printk("%s: LoadPacer: load values: A ctr1 %#x ctr2 %#x\n",ADAPTER_ID,
	 board->ctr1, board->ctr2);
#endif

  mask = C2 + MODE2 + LSBFIRST;
  outb_p(mask, COUNTERA_CONTROL); 

  bData = (BYTE) (board->ctr2 & 0xff);
  outb_p(bData, COUNTERA_2_DATA); 

  bData = (BYTE) (board->ctr2 >> 8);
  outb_p(bData, COUNTERA_2_DATA); 

  mask = C1 + MODE2 + LSBFIRST;
  outb_p(mask, COUNTERA_CONTROL); 

  bData = (board->ctr1 & 0xff);
  outb_p(bData, COUNTERA_1_DATA); 

  bData = (board->ctr1 >> 8) & 0xff;
  outb_p(bData, COUNTERA_1_DATA); 

  sti();
}

/***************************************************************************
 *
 * Load value into Counter 0  XXXX    Mode    MSB     LSB
 *                            Byte 3  Byte 2  Byte 1  Byte 0
 *
 ***************************************************************************/

static void LoadCounter0( LONG value )
{
  BYTE mask;
  BYTE bData;

  cli();

  /* Write the value into Counter 0 Mode 2 */
  
#ifdef DEBUG
  printk("%s: LoadCounter0: load value %#lx into Counter 0.\n",ADAPTER_ID,
    value);
#endif

  /* the mode is in the thrid byte */

  mask = 0xff & (value >> 16);
  mask += C0 + LSBFIRST;
  outb_p(mask, COUNTERA_CONTROL); 

  if ( value & 0xff000000 ) {
    /* load control word only */
    return;
  } else {
    /* LSB in the first byte of value */

    bData = (BYTE) (value & 0xff);
    outb_p(bData, COUNTERA_0_DATA); 

    /* MSB in the second byte of value */

    bData = (BYTE) ((value >> 8) & 0xff);
    outb_p(bData, COUNTERA_0_DATA); 
  }
  sti();
}

/***************************************************************************
 *
 * Set which channels read() is interested in.
 *
 ***************************************************************************/

static void SetChannelMux(BYTE lowChan, BYTE highChan)
{
  BYTE channelMask;

  channelMask = (highChan << 4) | lowChan;
  outb_p(channelMask, MUX_SCAN_LIMITS);
  
#ifdef DEBUG
  printk("%s: SetChannelMux: lowChan = %d  highChan = %d\n",
	 ADAPTER_ID, lowChan, highChan);
#endif

  return;
}

/***************************************************************************
 *
 * Turn off pacer timer chip.
 *
 ***************************************************************************/

static void StopPacer()
{
  BYTE bReg;

  cli();

#ifdef DEBUG
  printk("%s: C: StopPacer \n", ADAPTER_ID);
#endif

  /* Zero words passed to this routine is used to shutdown the DMA
       controller for the card.                                           */

  /* Note that this routine lets the card sending an interrupt.
   * We set Chan.pacer = FALSE to inform das0_ReadInterrupt.
   */
  bReg = inb_p(CNTRL_REG);
  bReg &= ~(CNTL_INTE | CNTL_DMA );
  outb_p(bReg, CNTRL_REG);
  disable_dma(DAS_DMA);

#ifdef DEBUG
  printk("%s: C: CNTRL_REG: %#2x \n", ADAPTER_ID, (int)bReg);
#endif

  /* The clock may run forever */

  Chan.pacer = FALSE ;

  sti();
}

static void StartPacer()
{
#ifdef DEBUG
  printk("%s: C: StartPacer \n", ADAPTER_ID);
#endif

  buf_reset();
  buf_check_len();
  {
    BYTE bReg = 0x0 ;

    SetGains((BYTE)Chan.gain);
    SetChannelMux(Chan.lowScan, Chan.highScan);
    Chan.nScan = Chan.highScan - Chan.lowScan + 1 ;
    Chan.mode = ADC_PACER_CLOCK ;
    Chan.pacer = TRUE ;
    /* Disable pacer if in pacer mode */
    /* software triggered A/D only */ 
    bReg = DAS_IRQ << 4 ;
    outb_p(bReg,CNTRL_REG);

    outb_p(0x40,CONVERT_DIS_REG); /* Disable Convertions */
    outb_p(0x40,BURST_MODE_REG);  /* Turns Burst Mode on */
    bReg = inb_p(BURST_STAT_REG);
    LoadPacer(&BoardData);
    BoardData.pacerReg |= 0x1 ;
#ifdef DEBUG
    printk("%s: C: start_pacer:  PacerREg = %#x\n",ADAPTER_ID,
	(int)BoardData.pacerReg);
#endif
    outb_p(BoardData.pacerReg,PACER_CLOCK_REG);
    cli();
    bReg= (BYTE) inb_p(MUX_SCAN_LIMITS); /* Read/Write clears FIFO */
#ifdef DEBUG
    printk("%s: C: start_pacer:  MUXval = %#x\n",ADAPTER_ID, (int)bReg);
#endif
    outb_p(bReg,MUX_SCAN_LIMITS);
    outb_p(0x0,CONVERT_DIS_REG); /* Enable Conversions  to the A/D */

    /* With the DMA controller now setup and active, we can tell the card
       to use the DMA transfer mode.                                      */
    bReg = DAS_IRQ << 4 ;
    bReg |= ( (DAS_IRQ << 4) | CNTL_DMA | CNTL_INTE| CNTL_TS0 | CNTL_TS1 );
    outb_p(bReg, CNTRL_REG);

    doDMA();

    return ;
  }
}

/***************************************************************************
 *
 * Block copy of data samples from data register to kernel memory.
 *
 ***************************************************************************/

static void doDMA()
{
  unsigned long flags;

    /* Ok, we have been asked to grab a block of data with the DMA */

    save_flags(flags);
    cli();

    /* The DMA controller has to be setup before the DAS-1600 is told
       to use DMA transfer mode.  So, setup the DMA first.  DMA controller
       must be off before it can be setup.                                 */
    buf_load_dma_controller();

    outb_p(0x0, STATUS_REG);

#ifdef DEBUG
    printk("%s: ?: doDMA(): DMA triggered and INT is enabled\n",ADAPTER_ID);
#endif

    restore_flags(flags);
  return;
}

