/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper
 * Copyright (C) 1995  Ricky Beam, Rob Cox
 * Copyright (C) 1996  Joerg Weule
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/*
 * adc.h
 */

/*  ioctl() values */

#define ADC_SET_GAINS       1
#define ADC_SET_PACER_FREQ  2
#define ADC_GET_PACER_FREQ  3
#define ADC_STOP_PACER      4
#define CTR0                5
#define COUNTER0            6
#define ADC_DIO_PRESET      7
#define DIO_SET_MODE        8
#define DIO_SET_DIRECTION   9
#define ADC_SETMUX_LIMITS  10
#define ADC_SET_MODE       11
#define ADC_SET_DMA_LEN    12
#define ADC_DA0            13
#define ADC_DA1            14
#define ADC_START_PACER    15
#define ADC_SET_CELL_LEN   16
#define ADC_GET_CELL_LEN   17
#define ADC_GET_DMA_LEN    18
#define ADC_DO_BLOCK       19

/* open mode values */

#define ADC_SOFT_TRIGGER         (0x0)
#define ADC_EXTERNAL_TRIGGER     (0x2)
#define ADC_PACER_CLOCK          (0x4)
#define ADC_PACER_EXTERN_TRIG    (0x6)

/* ioctl(COUNTER0) mode values */

#define CTR0_MODE0 (0x0 << 17)
#define CTR0_MODE1 (0x1 << 17)
#define CTR0_MODE2 (0x2 << 17)
#define CTR0_MODE3 (0x3 << 17)
#define CTR0_MODE4 (0x4 << 17)
#define CTR0_MODE5 (0x5 << 17)
#define CTR0_NOLOAD (0xff000000)

/*  General definitions */

#define MAX_FREQ 160000
#define BYTE unsigned char
#define WORD unsigned short
#define LONG unsigned long

/* Programmable Range and Gain Settings */

#define BP_10_00V  0x00           /* +/- 10V      */
#define BP_5_00V   0x01           /* +/-  5V      */
#define BP_2_50V   0x02           /* +/-  2.5V    */
#define UP_10_00V  0x00           /* 0 - 10V      */
#define UP_5_00V   0x01           /* 0 - 5V       */
#define UP_2_50V   0x02           /* 0 - 2.5V     */

/* digital output */
#define DIO_0 0x01
#define DIO_1 0x02
#define DIO_2 0x04
#define DIO_3 0x08
#define DIO_4 0x10
#define DIO_5 0x20
#define DIO_6 0x40
#define DIO_7 0x80

/* Digital I/O Modes */ 
#define MODE_IO            0
#define MODE_STROBE_IO     1
#define MODE_BIDIRECTIONAL 2

/* Digital I/O Direction Settings */
#define PORT_INPUT         1
#define PORT_OUTPUT        0
#define HIGH_PORT_INPUT    2
#define HIGH_PORT_OUTPUT   0
#define LOW_PORT_INPUT     1
#define LOW_PORT_OUTPPUT   0

/* This really COULD/SHOULD be a library */
__inline void ConvertData(long NumPoints, WORD *ADData, int *ChanTags) {
  
  int i;
  
  for (i=0;i<NumPoints;i++) {
    if(ChanTags) {
      ChanTags[i] = (ADData[i] & 0xf);
    }
    ADData[i] >>= 4;
  }
}

__inline void RawToReal(long NumPoints, WORD *RawData, float *RealData) {

  int i;

  for (i=0;i<NumPoints;i++) {
    RealData[i] = (RawData[i] - 2048.0) * (10.0 / 4095.0);
  }
}

