/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper
 * Copyright (C) 1996  Joerg Weule
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/***************************************************************************
 *
 *  adcread.c
 *
 *  This program is used to test the CIO-DAC 16/330 Anolog to Digitil
 *  Linux loadable module(adc).
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "adc.h"

/***************************************************************************
 *
 *  Global Data
 *
 ***************************************************************************/

char *DevName = "/dev/adc00";
int  Gain     = UP_5_00V;
int  Mode     = ADC_SOFT_TRIGGER;
int  Count    = 1;
int  NoStop   = 0;
int  Print    = 1;
int  Freq     = 100000;
int  Convert  = 1;
int  Real     = 1;
int  Chans    = 1;
int  dmalen   = 0;
int  da0      = -1 ;
int  da1      = -1 ;

/***************************************************************************
 *
 *  Show user how the command line is used.
 *
 ***************************************************************************/

void Usage( void )
{
    fprintf(stderr, "\n\
Usage: adcread \'options\'\n\
Options:\n\
              [-dev /dev/adc##]  - Specify device file.\n
              [-gain (0-8)]      - Voltage gain(see manual)\n
              [-ct ##]           - Number of samples to read\n
              [-soft]            - Use software triggers\n
              [-extern]          - Use extnernal clock triggers\n
              [-pacer]           - Use pacer clock triggers\n
              [-noprint]         - Don't print samples\n
              [-nostop]          - Sample forever\n
              [-dmalen ##]       - length of DMA\n
              [-chans ##]        - number of channels\n
              [-freq ##]         - Sample Frequency used with pacer triggers\n
              [-da0 ##]          - D/A chanel 0]\n
              [-da1 ##]          - D/A chanel 0]\n
\n");
    exit(1);
}

/***************************************************************************
 *
 *  Process command line switches
 *
 ***************************************************************************/

void DoCommandLine(int argc, char **argv)
{
int i;

i = 1;

while (i < argc) {
  if (strcmp(argv[i], "-dev") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      DevName = argv[i];
  } else if (strcmp(argv[i], "-range") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      Gain = atoi(argv[i]);
  } else if (strcmp(argv[i], "-ct") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      Count = atoi(argv[i]);
  } else if (strcmp(argv[i], "-soft") == 0) {
    Mode = ADC_SOFT_TRIGGER;
  } else if (strcmp(argv[i], "-pacer") == 0) {
    Mode = ADC_PACER_CLOCK;
  } else if (strcmp(argv[i], "-extern") == 0) {
    Mode = ADC_EXTERNAL_TRIGGER;
  } else if (strcmp(argv[i], "-noprint") == 0) {
    Print = 0;
  } else if (strcmp(argv[i], "-print") == 0) {
    Print = 1;
  } else if (strcmp(argv[i], "-noconvert") == 0) {
    Convert = 0;
  } else if (strcmp(argv[i], "-convert") == 0) {
    Convert = 1;
  } else if (strcmp(argv[i], "-nostop") == 0) {
    NoStop = 1;
  } else if (strcmp(argv[i], "-chans") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      Chans = atoi(argv[i]);
  } else if (strcmp(argv[i], "-freq") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      Freq = atoi(argv[i]);
  } else if (strcmp(argv[i], "-da0") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      da0 = atoi(argv[i]);
  } else if (strcmp(argv[i], "-da1") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      da1 = atoi(argv[i]);
  } else if (strcmp(argv[i], "-dmalen") == 0) {
    i++;
    if (i == argc)
      Usage();
    else 
      dmalen = atoi(argv[i]);
  } else {
    Usage();
  }
  i++;
}

}

/***************************************************************************
 *
 *  Main
 *
 ***************************************************************************/

int main(int argc, char **argv)
{
  int   i;
  int	fd;
  WORD  data[300000];
  float rdata[300000];
  int   chanel[300000];
  int   bytesRead;

  DoCommandLine(argc, argv);
	
  /* Open the device file for the counter
   */

  if ((fd = open(DevName, Mode )) < 0) {
    perror(DevName);
    printf("error opening device %s\n", DevName);
    exit(2);
  }

  /* Set DA */
  if ( da0 >= 0 ) {
    ioctl(fd,ADC_DA0,da0);
  }
  if ( da1 >= 0 ) {
    ioctl(fd,ADC_DA1,da1);
  }

  /* Set voltage gain */

  printf("Setting input range to %d. Look at manual.\n", Gain);
  ioctl(fd, ADC_SET_GAINS, Gain);

  /* program counter 0 */
  ioctl(fd, COUNTER0, CTR0_MODE2 | 10) ;
  ioctl(fd, CTR0, 0x1);
 
  /* Set sampling frequency */
  printf("Requested frequency at %d Hz.\n", Freq);
  ioctl(fd, ADC_SET_PACER_FREQ, Freq);
  ioctl(fd, ADC_GET_PACER_FREQ, &Freq);
  printf("Actual frequency set to %d Hz.\n", Freq);

  ioctl(fd, ADC_SETMUX_LIMITS, Chans-1);
  ioctl(fd, ADC_SET_DMA_LEN, (void*)dmalen);

  if ( Mode == ADC_PACER_CLOCK ) ioctl(fd, ADC_START_PACER , 0 );

  do {

    bytesRead = read(fd, data, Count);
    if ( 0==Print ) printf("main() in adcread.c: read()= %d \n",bytesRead);
    else {
      bytesRead /= 2 ;

      if(Convert)
        ConvertData(bytesRead, data, chanel);

      if(Real)
        RawToReal(bytesRead, data, rdata);

      if (Print==1) {
        for (i = 0; i < bytesRead; i++){ 
          printf("Sample %6d = (%6d) %6x ==> %9.5f , chanel = %d\n",
            i, data[i], data[i], rdata[i], chanel[i]);
        }
      }
    }
  } while (NoStop);
  printf("%d samples read\n", bytesRead);
  close(fd);
  return 0;
}

